# Copyright (c) 2026 Pantheon Lab Pte Ltd
# Licensed under the Business Source License 1.1 (see LICENSE.cloud)
"""Navil Cloud — web dashboard and REST API for Navil."""

from __future__ import annotations
